@@ -273,6 +273,37 @@ setMethod("commits",
                   stop("'n' must be integer")
               }
 
+              if (is_shallow(repo)) {
+                  ## FIXME: Remove this if-statement when libgit2
+                  ## supports shallow clones, see #219.  Note: This
+                  ## workaround does not use the 'topological', 'time'
+                  ## and 'reverse' flags.
+
+                  ## List to hold result
+                  result <- list()
+
+                  ## Get latest commit
+                  x <- lookup(repo, branch_target(head(repo)))
+
+                  ## Repeat until no more parent commits
+                  repeat {
+                      if (n == 0) {
+                          break
+                      } else if (n > 0) {
+                          n <- n - 1
+                      }
+
+                      if (is.null(x))
+                          break
+                      result <- append(result, x)
+
+                      ## Get parent to commit
+                      x <- tryCatch(parents(x)[[1]], error = function(e) NULL)
+                  }
+
+                  return(result)
+              }
+
               .Call(git2r_revwalk_list, repo, topological, time, reverse, n)
           }
 )
