@@ -126,6 +126,10 @@ int git2r_arg_check_credentials(SEXP arg)
             return -1;
         if (git2r_arg_check_string(GET_SLOT(arg, Rf_install("password"))))
             return -1;
+    } else if (0 == strcmp(CHAR(STRING_ELT(class_name, 0)), "cred_token")) {
+        /* Check token */
+        if (git2r_arg_check_string(GET_SLOT(arg, Rf_install("token"))))
+            return -1;
     } else if (0 == strcmp(CHAR(STRING_ELT(class_name, 0)), "cred_user_pass")) {
         /* Check username and password */
         if (git2r_arg_check_string(GET_SLOT(arg, Rf_install("username"))))
@@ -358,7 +362,7 @@ int git2r_arg_check_real(SEXP arg)
     if (R_NilValue == arg
         || !isReal(arg)
         || 1 != length(arg)
-        || NA_REAL == REAL(arg)[0])
+        || !R_finite(REAL(arg)[0]))
         return -1;
     return 0;
 }
