@@ -22,15 +22,7 @@
 #include "git2r_clone.h"
 #include "git2r_cred.h"
 #include "git2r_error.h"
-
-/**
- * Data structure to hold information when performing clone.
- */
-typedef struct {
-    int received_progress;
-    int received_done;
-    SEXP credentials;
-} git2r_clone_data;
+#include "git2r_transfer.h"
 
 /**
  * Show progress of clone
@@ -45,7 +37,7 @@ static int git2r_clone_progress(
     void *payload)
 {
     int kbytes = progress->received_bytes / 1024;
-    git2r_clone_data *pd = (git2r_clone_data*)payload;
+    git2r_transfer_data *pd = (git2r_transfer_data*)payload;
 
     if (progress->received_objects < progress->total_objects) {
         int received_percent =
@@ -71,39 +63,14 @@ static int git2r_clone_progress(
     return 0;
 }
 
-/**
- * Callback if the remote host requires authentication in order to
- * connect to it
- *
- * @param cred The newly created credential object.
- * @param url The resource for which we are demanding a credential.
- * @param user_from_url The username that was embedded in a "user@host"
- * remote url, or NULL if not included.
- * @param allowed_types A bitmask stating which cred types are OK to return.
- * @param payload The payload provided when specifying this callback.
- * @return 0 on success, else -1.
- */
-int git2r_clone_cred_acquire(
-    git_cred **cred,
-    const char *url,
-    const char *username_from_url,
-    unsigned int allowed_types,
-    void *payload)
-{
-    return git2r_cred_acquire_cb(
-        cred,
-        url,
-        username_from_url,
-        allowed_types,
-        ((git2r_clone_data*)payload)->credentials);
-}
-
 /**
  * Clone a remote repository
  *
  * @param url the remote repository to clone
  * @param local_path local directory to clone to
  * @param bare Create a bare repository.
+ * @param branch The name of the branch to checkout. Default is NULL
+ *        which means to use the remote's default branch.
  * @param credentials The credentials for remote repository access.
  * @param progress show progress
  * @return R_NilValue
@@ -112,6 +79,7 @@ SEXP git2r_clone(
     SEXP url,
     SEXP local_path,
     SEXP bare,
+    SEXP branch,
     SEXP credentials,
     SEXP progress)
 {
@@ -119,30 +87,35 @@ SEXP git2r_clone(
     git_repository *repository = NULL;
     git_clone_options clone_opts = GIT_CLONE_OPTIONS_INIT;
     git_checkout_options checkout_opts = GIT_CHECKOUT_OPTIONS_INIT;
-    git2r_clone_data payload = {0, 0, R_NilValue};
+    git2r_transfer_data payload = GIT2R_TRANSFER_DATA_INIT;
 
     if (git2r_arg_check_string(url))
-        git2r_error(git2r_err_string_arg, __func__, "url");
+        git2r_error(__func__, NULL, "'url'", git2r_err_string_arg);
     if (git2r_arg_check_string(local_path))
-        git2r_error(git2r_err_string_arg, __func__, "local_path");
+        git2r_error(__func__, NULL, "'local_path'", git2r_err_string_arg);
     if (git2r_arg_check_logical(bare))
-        git2r_error(git2r_err_logical_arg, __func__, "bare");
+        git2r_error(__func__, NULL, "'bare'", git2r_err_logical_arg);
+    if (branch != R_NilValue && git2r_arg_check_string(branch))
+        git2r_error(__func__, NULL, "'branch'", git2r_err_string_arg);
     if (git2r_arg_check_credentials(credentials))
-        git2r_error(git2r_err_credentials_arg, __func__, "credentials");
+        git2r_error(__func__, NULL, "'credentials'", git2r_err_credentials_arg);
     if (git2r_arg_check_logical(progress))
-        git2r_error(git2r_err_logical_arg, __func__, "progress");
+        git2r_error(__func__, NULL, "'progress'", git2r_err_logical_arg);
 
     checkout_opts.checkout_strategy = GIT_CHECKOUT_SAFE;
     clone_opts.checkout_opts = checkout_opts;
     payload.credentials = credentials;
-    clone_opts.remote_callbacks.payload = &payload;
-    clone_opts.remote_callbacks.credentials = &git2r_clone_cred_acquire;
+    clone_opts.fetch_opts.callbacks.payload = &payload;
+    clone_opts.fetch_opts.callbacks.credentials = &git2r_cred_acquire_cb;
 
     if (LOGICAL(bare)[0])
         clone_opts.bare = 1;
 
+    if (branch != R_NilValue)
+        clone_opts.checkout_branch = CHAR(STRING_ELT(branch, 0));
+
     if (LOGICAL(progress)[0]) {
-        clone_opts.remote_callbacks.transfer_progress = &git2r_clone_progress;
+        clone_opts.fetch_opts.callbacks.transfer_progress = &git2r_clone_progress;
         Rprintf("cloning into '%s'...\n", CHAR(STRING_ELT(local_path, 0)));
     }
 
@@ -154,8 +127,12 @@ SEXP git2r_clone(
     if (repository)
         git_repository_free(repository);
 
-    if (GIT_OK != err)
-        git2r_error(git2r_err_from_libgit2, __func__, giterr_last()->message);
+    if (err)
+        git2r_error(
+            __func__,
+            giterr_last(),
+            git2r_err_unable_to_authenticate,
+            NULL);
 
     return R_NilValue;
 }
