@@ -1,6 +1,6 @@
 /*
  *  git2r, R bindings to the libgit2 library.
- *  Copyright (C) 2013-2015 The git2r contributors
+ *  Copyright (C) 2013-2016 The git2r contributors
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -79,31 +79,33 @@ SEXP git2r_remote_add(SEXP repo, SEXP name, SEXP url)
  *
  * Based on the libgit2 network/fetch.c example.
  *
- * @refname The name of the remote
- * @a The previous position of branch
- * @b The new position of branch
- * @data Callback data. Not used.
+ * @param refname The name of the remote
+ * @param a The previous position of branch
+ * @param b The new position of branch
+ * @param payload Callback data.
  * @return 0
  */
 static int git2r_update_tips_cb(
     const char *refname,
     const git_oid *a,
     const git_oid *b,
-    void *data)
+    void *payload)
 {
-    char a_str[GIT_OID_HEXSZ + 1], b_str[GIT_OID_HEXSZ + 1];
+    git2r_transfer_data *cb_data = (git2r_transfer_data*)payload;
 
-    GIT_UNUSED(data);
+    if (cb_data->verbose) {
+        char b_str[GIT_OID_HEXSZ + 1];
+        git_oid_fmt(b_str, b);
+        b_str[GIT_OID_HEXSZ] = '\0';
 
-    git_oid_fmt(b_str, b);
-    b_str[GIT_OID_HEXSZ] = '\0';
-
-    if (git_oid_iszero(a)) {
-        Rprintf("[new]     %.20s %s\n", b_str, refname);
-    } else {
-        git_oid_fmt(a_str, a);
-        a_str[GIT_OID_HEXSZ] = '\0';
-        Rprintf("[updated] %.10s..%.10s %s\n", a_str, b_str, refname);
+        if (git_oid_iszero(a)) {
+            Rprintf("[new]     %.20s %s\n", b_str, refname);
+        } else {
+            char a_str[GIT_OID_HEXSZ + 1];
+            git_oid_fmt(a_str, a);
+            a_str[GIT_OID_HEXSZ] = '\0';
+            Rprintf("[updated] %.10s..%.10s %s\n", a_str, b_str, refname);
+        }
     }
 
     return 0;
@@ -116,13 +118,18 @@ static int git2r_update_tips_cb(
  * @param name The name of the remote to fetch from
  * @param credentials The credentials for remote repository access.
  * @param msg The one line long message to be appended to the reflog
+ * @param verbose Print information each time a reference is updated locally.
+ * @param refspecs The refspecs to use for this fetch. Pass R_NilValue
+ *        to use the base refspecs.
  * @return R_NilValue
  */
 SEXP git2r_remote_fetch(
     SEXP repo,
     SEXP name,
     SEXP credentials,
-    SEXP msg)
+    SEXP msg,
+    SEXP verbose,
+    SEXP refspecs)
 {
     int err;
     SEXP result = R_NilValue;
@@ -131,6 +138,7 @@ SEXP git2r_remote_fetch(
     git_repository *repository = NULL;
     git_fetch_options fetch_opts = GIT_FETCH_OPTIONS_INIT;
     git2r_transfer_data payload = GIT2R_TRANSFER_DATA_INIT;
+    git_strarray refs = {0};
 
     if (git2r_arg_check_string(name))
         git2r_error(__func__, NULL, "'name'", git2r_err_string_arg);
@@ -138,6 +146,10 @@ SEXP git2r_remote_fetch(
         git2r_error(__func__, NULL, "'credentials'", git2r_err_credentials_arg);
     if (git2r_arg_check_string(msg))
         git2r_error(__func__, NULL, "'msg'", git2r_err_string_arg);
+    if (git2r_arg_check_logical(verbose))
+        git2r_error(__func__, NULL, "'verbose'", git2r_err_logical_arg);
+    if (refspecs != R_NilValue && git2r_arg_check_string_vec(refspecs))
+        git2r_error(__func__, NULL, "'refspecs'", git2r_err_string_vec_arg);
 
     repository = git2r_repository_open(repo);
     if (!repository)
@@ -147,11 +159,38 @@ SEXP git2r_remote_fetch(
     if (err)
         goto cleanup;
 
+    if (refspecs != R_NilValue) {
+        size_t i, len;
+
+        /* Count number of non NA values */
+        len = length(refspecs);
+        for (i = 0; i < len; i++)
+            if (NA_STRING != STRING_ELT(refspecs, i))
+                refs.count++;
+
+        if (refs.count) {
+            /* Allocate the strings in refs */
+            refs.strings = malloc(refs.count * sizeof(char*));
+            if (!refs.strings) {
+                giterr_set_str(GITERR_NONE, git2r_err_alloc_memory_buffer);
+                err = GIT_ERROR;
+                goto cleanup;
+            }
+
+            /* Populate the strings in refs */
+            for (i = 0; i < refs.count; i++)
+                if (NA_STRING != STRING_ELT(refspecs, i))
+                    refs.strings[i] = (char *)CHAR(STRING_ELT(refspecs, i));
+        }
+    }
+
+    if (LOGICAL(verbose)[0])
+        payload.verbose = 1;
     payload.credentials = credentials;
     fetch_opts.callbacks.payload = &payload;
     fetch_opts.callbacks.credentials = &git2r_cred_acquire_cb;
     fetch_opts.callbacks.update_tips = &git2r_update_tips_cb;
-    err = git_remote_fetch(remote, NULL, &fetch_opts, CHAR(STRING_ELT(msg, 0)));
+    err = git_remote_fetch(remote, &refs, &fetch_opts, CHAR(STRING_ELT(msg, 0)));
     if (err)
         goto cleanup;
 
@@ -160,6 +199,9 @@ SEXP git2r_remote_fetch(
     git2r_transfer_progress_init(stats, result);
 
 cleanup:
+    if (refs.strings)
+        free(refs.strings);
+
     if (remote) {
         if (git_remote_connected(remote))
             git_remote_disconnect(remote);
@@ -434,7 +476,7 @@ SEXP git2r_remote_ls(SEXP name, SEXP repo, SEXP credentials)
     callbacks.payload = &payload;
     callbacks.credentials = &git2r_cred_acquire_cb;
 
-    err = git_remote_connect(remote, GIT_DIRECTION_FETCH, &callbacks);
+    err = git_remote_connect(remote, GIT_DIRECTION_FETCH, &callbacks, NULL);
     if (err)
         goto cleanup;
 
