@@ -19,10 +19,15 @@
 #include <Rdefines.h>
 
 #include "git2r_arg.h"
+#include "git2r_blob.h"
+#include "git2r_commit.h"
 #include "git2r_error.h"
 #include "git2r_repository.h"
 #include "git2r_signature.h"
 #include "git2r_tag.h"
+#include "git2r_tree.h"
+
+#include "util.h"
 
 /**
  * Init slots in S4 class git_tag
@@ -77,22 +82,22 @@ SEXP git2r_tag_create(SEXP repo, SEXP name, SEXP message, SEXP tagger)
     git_object *target = NULL;
 
     if (git2r_arg_check_string(name))
-        git2r_error(git2r_err_string_arg, __func__, "name");
+        git2r_error(__func__, NULL, "'name'", git2r_err_string_arg);
     if (git2r_arg_check_string(message))
-        git2r_error(git2r_err_string_arg, __func__, "message");
+        git2r_error(__func__, NULL, "'message'", git2r_err_string_arg);
     if (git2r_arg_check_signature(tagger))
-        git2r_error(git2r_err_signature_arg, __func__, "tagger");
+        git2r_error(__func__, NULL, "'tagger'", git2r_err_signature_arg);
 
     repository = git2r_repository_open(repo);
     if (!repository)
-        git2r_error(git2r_err_invalid_repository, __func__, NULL);
+        git2r_error(__func__, NULL, git2r_err_invalid_repository, NULL);
 
     err = git2r_signature_from_arg(&sig_tagger, tagger);
-    if (GIT_OK != err)
+    if (err)
         goto cleanup;
 
     err = git_revparse_single(&target, repository, "HEAD^{commit}");
-    if (GIT_OK != err)
+    if (err)
         goto cleanup;
 
     err = git_tag_create(
@@ -103,11 +108,11 @@ SEXP git2r_tag_create(SEXP repo, SEXP name, SEXP message, SEXP tagger)
         sig_tagger,
         CHAR(STRING_ELT(message, 0)),
         0);
-    if (GIT_OK != err)
+    if (err)
         goto cleanup;
 
     err = git_tag_lookup(&tag, repository, &oid);
-    if (GIT_OK != err)
+    if (err)
         goto cleanup;
 
     PROTECT(result = NEW_OBJECT(MAKE_CLASS("git_tag")));
@@ -129,12 +134,100 @@ cleanup:
     if (R_NilValue != result)
         UNPROTECT(1);
 
-    if (GIT_OK != err)
-        git2r_error(git2r_err_from_libgit2, __func__, giterr_last()->message);
+    if (err)
+        git2r_error(__func__, giterr_last(), NULL, NULL);
 
     return result;
 }
 
+/**
+ * Data structure to hold information when iterating over tags.
+ */
+typedef struct {
+    size_t n;
+    git_repository *repository;
+    SEXP repo;
+    SEXP tags;
+} git2r_tag_foreach_cb_data;
+
+/**
+ * Invoked 'callback' for each tag
+ *
+ * @param name The name of the tag
+ * @param oid The id of the tag
+ * @param payload Payload data passed to 'git_tag_foreach'
+ * @return 0 on success, else error code
+ */
+static int git2r_tag_foreach_cb(const char *name, git_oid *oid, void *payload)
+{
+    int err = 0;
+    git_object *object = NULL;
+    git2r_tag_foreach_cb_data *cb_data = (git2r_tag_foreach_cb_data*)payload;
+
+    /* Check if we have a list to populate */
+    if (R_NilValue != cb_data->tags) {
+        int skip = 0;
+        SEXP item;
+
+        err = git_object_lookup(&object, cb_data->repository, oid, GIT_OBJ_ANY);
+        if (err)
+            goto cleanup;
+
+        switch (git_object_type(object)) {
+        case GIT_OBJ_COMMIT:
+            SET_VECTOR_ELT(
+                cb_data->tags,
+                cb_data->n,
+                item = NEW_OBJECT(MAKE_CLASS("git_commit")));
+            git2r_commit_init((git_commit*)object, cb_data->repo, item);
+            break;
+        case GIT_OBJ_TREE:
+            SET_VECTOR_ELT(
+                cb_data->tags,
+                cb_data->n,
+                item = NEW_OBJECT(MAKE_CLASS("git_tree")));
+            git2r_tree_init((git_tree*)object, cb_data->repo, item);
+            break;
+        case GIT_OBJ_BLOB:
+            SET_VECTOR_ELT(
+                cb_data->tags,
+                cb_data->n,
+                item = NEW_OBJECT(MAKE_CLASS("git_blob")));
+            git2r_blob_init((git_blob*)object, cb_data->repo, item);
+            break;
+        case GIT_OBJ_TAG:
+            SET_VECTOR_ELT(
+                cb_data->tags,
+                cb_data->n,
+                item = NEW_OBJECT(MAKE_CLASS("git_tag")));
+            git2r_tag_init((git_tag*)object, cb_data->repo, item);
+            break;
+        default:
+            git2r_error(__func__, NULL, git2r_err_object_type, NULL);
+        }
+
+
+        if (git__prefixcmp(name, "refs/tags/") == 0)
+            skip = strlen("refs/tags/");
+        SET_STRING_ELT(
+            getAttrib(cb_data->tags, R_NamesSymbol),
+            cb_data->n,
+            mkChar(name + skip));
+
+        if (object)
+            git_object_free(object);
+        object = NULL;
+    }
+
+    cb_data->n += 1;
+
+cleanup:
+    if (object)
+        git_object_free(object);
+
+    return err;
+}
+
 /**
  * Get all tags that can be found in a repository.
  *
@@ -144,62 +237,45 @@ cleanup:
 SEXP git2r_tag_list(SEXP repo)
 {
     int err;
-    SEXP list = R_NilValue;
+    SEXP result = R_NilValue;
+    git2r_tag_foreach_cb_data cb_data = {0, NULL, R_NilValue, R_NilValue};
     git_repository *repository;
-    git_reference* reference = NULL;
-    git_tag *tag = NULL;
-    git_strarray tag_names = {0};
-    size_t i;
 
     repository = git2r_repository_open(repo);
     if (!repository)
-        git2r_error(git2r_err_invalid_repository, __func__, NULL);
+        git2r_error(__func__, NULL, git2r_err_invalid_repository, NULL);
+
+    /* Count number of tags before creating the list */
+    err = git_tag_foreach(repository, &git2r_tag_foreach_cb, &cb_data);
+    if (err) {
+        if (GIT_ENOTFOUND == err) {
+            err = 0;
+            PROTECT(result = allocVector(VECSXP, 0));
+            setAttrib(result, R_NamesSymbol, allocVector(STRSXP, 0));
+        }
 
-    err = git_tag_list(&tag_names, repository);
-    if (GIT_OK != err)
         goto cleanup;
-
-    PROTECT(list = allocVector(VECSXP, tag_names.count));
-
-    for(i = 0; i < tag_names.count; i++) {
-        SEXP sexp_tag;
-        const git_oid *oid;
-
-        err = git_reference_dwim(&reference, repository, tag_names.strings[i]);
-        if (GIT_OK != err)
-            goto cleanup;
-
-        oid = git_reference_target(reference);
-        err = git_tag_lookup(&tag, repository, oid);
-        if (GIT_OK != err)
-            goto cleanup;
-
-        SET_VECTOR_ELT(list, i, sexp_tag = NEW_OBJECT(MAKE_CLASS("git_tag")));
-        git2r_tag_init(tag, repo, sexp_tag);
-
-        git_tag_free(tag);
-        tag = NULL;
-        git_reference_free(reference);
-        reference = NULL;
     }
 
+    PROTECT(result = allocVector(VECSXP, cb_data.n));
+    setAttrib(result, R_NamesSymbol, allocVector(STRSXP, cb_data.n));
+
+    cb_data.n = 0;
+    cb_data.tags = result;
+    cb_data.repo = repo;
+    cb_data.repository = repository;
+
+    err = git_tag_foreach(repository, &git2r_tag_foreach_cb, &cb_data);
+
 cleanup:
-    git_strarray_free(&tag_names);
-
-    if (tag)
-        git_tag_free(tag);
-
-    if (reference)
-        git_reference_free(reference);
-
     if (repository)
         git_repository_free(repository);
 
-    if (R_NilValue != list)
+    if (R_NilValue != result)
         UNPROTECT(1);
 
-    if (GIT_OK != err)
-        git2r_error(git2r_err_from_libgit2, __func__, giterr_last()->message);
+    if (err)
+        git2r_error(__func__, giterr_last(), NULL, NULL);
 
-    return list;
+    return result;
 }
